# Display and run all Koenig SOF files in a folder chosen by the user.
# Author: Dave Greene (dvgrn@juno.com), 29 May 2006; updated 29 April 2007.
#         Updated again 1 April 2009 to work with Golly 2.0.
# Still need to add an ambitious (but not too ambitious!) pattern deletion system
# (probably throttle up and down depending on how many rows in the total pattern!)

from glife import *
import golly as g

import os
from os.path import join
from time import time

global display_step, basex
display_step=1
basex=999999

# ------------------------------------------------------------------------------

def flyover ():
   basedir=g.getstring("Enter path of folder to display: ")
   if basedir=="": basedir=os.path.join(os.path.join(golly.appdir() + "Scripts","Python"),"ObjectDB")
   
   g.new("HK Database Flyover")
   g.setcell(50,50,1) # lazy way to be sure of not getting degenerate getrect[] result
   g.setmag(g.getmag()-2)

   for root, dirs, files in os.walk(basedir):
      for name in files:
         # ignore hidden files (like .DS_Store on Mac)
         if name.endswith(".sof") and not name.startswith("."):
            fullname = join(root, name)
            runSOF(fullname)
      if "CVS" in dirs:
         dirs.remove("CVS")  # don't visit CVS directories

def runSOF (fname):
   global display_step, basex
   f=open(fname, 'r')
   count=0
   skipNobjects=0
   for chunk in f:
      # Python has some annoying trouble with Mac-style line endings,
      # so we have to make everything look right no matter what:
      # in Mac format, entire file will be here, separated by chr(10)
      for line in chunk.split(chr(10)):
         if line=="": continue
         count+=1
         if skipNobjects>0: # skip objects if S key pressed
            skipNobjects-=1
            continue
         g.show(fname + ", line " + str(count))
         bounds=g.getrect()
         x=int(bounds[0])
         y=int(bounds[1])
         w=int(bounds[2])
         h=int(bounds[3])
         # set x location so that column won't drift if bounding box changes
         if basex==999999: basex=x
         linedata=line.split(",")
         if len(linedata)==1:
            # need more vertical space here, since we don't know the object types
            # (could look at the filenames, but that's what the CSV database format is for.)
            end_y=showSOF(linedata[0], basex, y + h + 32)
         else:
            # modified SOF database has comma-separated sourcename, cells, period, vector
            if linedata[3]=="H0V0":
               end_y=showSOF(linedata[4], x, y + h + 10)
            elif linedata[3][-2:]=="V0":
               # objects are displayed reflected in Golly's x=y line,
               # so an HnV0 object actually shows up going north,
               # and should be offset so avoid collisions.
               # The calculation below sorts vertical-travelling spaceships
               # by speed, though 2c/5 spaceships show up with dangerously little room,
               # and there are a few collisions in the current sample database.
               end_y=showSOF(linedata[4], basex + 128*int(linedata[2]) / int(linedata[3][1:-2]) - 160, y + h + 10)
            else:
               end_y=showSOF(linedata[4], basex, y + h + 50)
         g.update()

         while not g.visrect( [x, end_y, 1, 1] ): # this didn't work reliably at low zooms (?!)
            # stop autoscrolling when new object's last row is on screen (or parallel)
            g.run(display_step)
            g.update()

            # autoscrolling happens here
            pos = g.getpos()
            g.setpos(pos[0],str(int(pos[1])+display_step))

            # keyboard handling
            while 1:
               ch=g.getkey()
               if ch=="+":
                  display_step = (display_step + 1)*2 - 1
               elif ch=="-":
                  if display_step != 1:
                     display_step = (display_step + 1)/2 - 1
               elif ch=="[":
                     g.setmag(g.getmag()-1)
               elif ch=="]":
                     g.setmag(g.getmag()+1)                     
               elif ch=="s" or ch=="S": skipNobjects=100
               elif ch=="": break
               else:
                  g.dokey(ch)
   f.close()
      
def showSOF (sofString, x, y):
   # process the SOF and produce an object at chosen location
   # current databases seem to work best with objects reflected in Golly's x=y
   cur_x = x
   cellState=1
   for ch in sofString:
      if ch=='-':
         cur_x=x
         y+=1
         cellState=1
      elif ch=='.':
         break
      else:
         runlength=ord(ch)-48
         while runlength>0:
            if cellState==1:
               g.setcell(cur_x, y, cellState)
               cur_x+=1
               runlength-=1
            else:
               cur_x+=runlength
               break
         cellState=1-cellState
   return y

# ------------------------------------------------------------------------------

# remember initial hashing state so we can restore it if changed by a pattern file
inithash = g.getoption("hashing")

# show status bar so user sees messages
initstatus = g.setoption("showstatusbar", True)

# hide other stuff to maximize the viewport
# (don't switch to fullscreen because user won't see pattern name in title)

initlayerbar = g.setoption("showlayerbar", False)
inittoolbar = g.setoption("showtoolbar", False)
initscripts = g.setoption("showscripts", False)
initpatterns = g.setoption("showpatterns", False)
initgrid = g.setoption("showgrid", False)

try:
   flyover()
except:
   g.show("")
   g.setoption("hashing", inithash)
   g.setoption("showstatusbar", initstatus)
   g.setoption("showlayerbar", initlayerbar)
   g.setoption("showtoolbar", inittoolbar)
   g.setoption("showscripts", initscripts)
   g.setoption("showpatterns", initpatterns)
   g.setoption("showgrid", initgrid)